{% load i18n horizon %}
<p class="well">
    {% blocktrans %}Select which type of job that you want to run.
      This choice will dictate which steps are required to successfully
      execute your job.
    {% endblocktrans %}
</p>
