{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans %}Add libraries to your job template.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Choose from the list of binaries and click "choose" to add the library to your job template.  This can be repeated for additional libraries.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}For Shell Action jobs, any required files beyond the main script may be added as "libraries".{% endblocktrans %}
</p>
</div>